#!/usr/bin/perl

# Crontab editor.   Uses crontab command to re-invoke the program editing
# the crontab file and adding/replacing an entry to it.


use Cwd;


if( my $newline = $ENV{CTABENTRY} ) {
    # we are in crontab mode
    my $tab = $ARGV[0];
    my @out;

    my($minute, $hour, $day, $month, $weekday, $command) =
                split(/\s+/,$newline,6);

    open( CRON, $tab ) || die "Can't open crontab file";
    while( <CRON> ) {
        push (@out, $_), next if /^#/;

        my(@res) = split(/\s+/,$_,6);
        my $com = $res[$#res];  # last element
        chomp $com;
        if( !$foundline && $com eq $command ) {
            if( $_ eq "$newline\n" ) {
                print STDERR "Cron line '$newline' already present\n";
                }
             else {
                print STDERR "Replacing cron line: '$_'\twith: '$newline'\n";
                push( @out, "$newline\n" );
                $changed++;
                }
            $foundline++;
            }
         else {
            push @out, $_;
            }
        }
    unless( $foundline ) {
        print STDERR "Adding cron line: '$newline'\n";
        push @out, "$newline\n";
        $changed++;
        }
    close CRON;

    if( $changed ) {
        open( COUT, ">$tab" ) || die "Can't write crontab file";
        print COUT @out;
        close COUT;
        }
    exit 0;
    }
        
# No environment variable so we are being called to add a line, found on
# stdin

my $cronline;

if( @ARGV ) {
    $cronline = join( '\t', @ARGV );
    }
 else {
    $cronline = <STDIN>;
    }
chomp $cronline;

my( @parts ) = split(/\s+/,$cronline,6);
if( @parts != 6 ) {
    die "Invalid crontab line $cronline\n";
    }

my $script = $0;
unless( $script =~ m#/# ) {
    # attach the current directory to it
    $script = getcwd . "/" . $script;
    }

$ENV{"EDITOR"} = "perl $script";
$ENV{"CTABENTRY"} = $cronline;
print STDERR "Invoking crontab to add entry:\n$cronline\n";
system( "crontab -e" );
